Files
TheRock/rocm-libraries/projects/hipblas/clients/include/blas2/testing_her_batched.hpp
T
derek.holloway 7502bf64a4
CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
Flatten subprojects into a single base monorepo
2026-09-02 16:34:47 -07:00

308 lines
12 KiB
C++

/* ************************************************************************
* Copyright (C) 2016-2025 Advanced Micro Devices, Inc. All rights reserved.
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* ************************************************************************ */
#include <fstream>
#include <iostream>
#include <stdlib.h>
#include <vector>
#include "testing_common.hpp"
/* ============================================================================================ */
using hipblasHerBatchedModel
= ArgumentModel<e_a_type, e_uplo, e_N, e_alpha, e_incx, e_lda, e_batch_count>;
inline void testname_her_batched(const Arguments& arg, std::string& name)
{
hipblasHerBatchedModel{}.test_name(arg, name);
}
template <typename T>
void testing_her_batched_bad_arg(const Arguments& arg)
{
using U = real_t<T>;
bool FORTRAN = arg.api == hipblas_client_api::FORTRAN;
auto hipblasHerBatchedFn
= FORTRAN ? hipblasHerBatched<T, U, true> : hipblasHerBatched<T, U, false>;
auto hipblasHerBatchedFn_64 = arg.api == FORTRAN_64 ? hipblasHerBatched_64<T, U, true>
: hipblasHerBatched_64<T, U, false>;
for(auto pointer_mode : {HIPBLAS_POINTER_MODE_HOST, HIPBLAS_POINTER_MODE_DEVICE})
{
hipblasLocalHandle handle(arg);
CHECK_HIPBLAS_ERROR(hipblasSetPointerMode(handle, pointer_mode));
hipblasFillMode_t uplo = HIPBLAS_FILL_MODE_UPPER;
int64_t N = 100;
int64_t lda = 100;
int64_t incx = 1;
int64_t batch_count = 2;
device_vector<U> d_alpha(1), d_zero(1);
const U h_alpha{1}, h_zero{0};
const U* alpha = &h_alpha;
const U* zero = &h_zero;
if(pointer_mode == HIPBLAS_POINTER_MODE_DEVICE)
{
CHECK_HIP_ERROR(hipMemcpy(d_alpha, alpha, sizeof(*alpha), hipMemcpyHostToDevice));
CHECK_HIP_ERROR(hipMemcpy(d_zero, zero, sizeof(*zero), hipMemcpyHostToDevice));
alpha = d_alpha;
zero = d_zero;
}
device_batch_matrix<T> dA(N, N, lda, batch_count);
device_batch_vector<T> dx(N, incx, batch_count);
DAPI_EXPECT(HIPBLAS_STATUS_NOT_INITIALIZED,
hipblasHerBatchedFn,
(nullptr,
uplo,
N,
alpha,
dx.ptr_on_device(),
incx,
dA.ptr_on_device(),
lda,
batch_count));
DAPI_EXPECT(HIPBLAS_STATUS_INVALID_VALUE,
hipblasHerBatchedFn,
(handle,
HIPBLAS_FILL_MODE_FULL,
N,
alpha,
dx.ptr_on_device(),
incx,
dA.ptr_on_device(),
lda,
batch_count));
DAPI_EXPECT(HIPBLAS_STATUS_INVALID_ENUM,
hipblasHerBatchedFn,
(handle,
(hipblasFillMode_t)HIPBLAS_OP_N,
N,
alpha,
dx.ptr_on_device(),
incx,
dA.ptr_on_device(),
lda,
batch_count));
DAPI_EXPECT(HIPBLAS_STATUS_INVALID_VALUE,
hipblasHerBatchedFn,
(handle,
uplo,
N,
nullptr,
dx.ptr_on_device(),
incx,
dA.ptr_on_device(),
lda,
batch_count));
if(pointer_mode == HIPBLAS_POINTER_MODE_HOST)
{
// For device mode in rocBLAS we don't have checks for dA, dx as we may be able to quick return
DAPI_EXPECT(
HIPBLAS_STATUS_INVALID_VALUE,
hipblasHerBatchedFn,
(handle, uplo, N, alpha, nullptr, incx, dA.ptr_on_device(), lda, batch_count));
DAPI_EXPECT(
HIPBLAS_STATUS_INVALID_VALUE,
hipblasHerBatchedFn,
(handle, uplo, N, alpha, dx.ptr_on_device(), incx, nullptr, lda, batch_count));
}
// With N == 0, can have all nullptrs
DAPI_CHECK(hipblasHerBatchedFn,
(handle, uplo, 0, nullptr, nullptr, incx, nullptr, lda, batch_count));
// With batch_count == 0, can have all nullptrs
DAPI_CHECK(hipblasHerBatchedFn, (handle, uplo, N, nullptr, nullptr, incx, nullptr, lda, 0));
// With alpha == 0, can have all nullptrs
DAPI_CHECK(hipblasHerBatchedFn,
(handle, uplo, N, zero, nullptr, incx, nullptr, lda, batch_count));
}
}
template <typename T>
void testing_her_batched(const Arguments& arg)
{
using U = real_t<T>;
bool FORTRAN = arg.api == hipblas_client_api::FORTRAN;
auto hipblasHerBatchedFn
= FORTRAN ? hipblasHerBatched<T, U, true> : hipblasHerBatched<T, U, false>;
auto hipblasHerBatchedFn_64 = arg.api == FORTRAN_64 ? hipblasHerBatched_64<T, U, true>
: hipblasHerBatched_64<T, U, false>;
hipblasFillMode_t uplo = char2hipblas_fill(arg.uplo);
int64_t N = arg.N;
int64_t incx = arg.incx;
int64_t lda = arg.lda;
int64_t batch_count = arg.batch_count;
size_t A_size = lda * N;
double hipblas_error_host{0}, hipblas_error_device{0};
U h_alpha = arg.get_alpha<U>();
hipblasLocalHandle handle(arg);
// argument sanity check, quick return if input parameters are invalid before allocating invalid
// memory
bool invalid_size = N < 0 || lda < N || lda < 1 || !incx || batch_count < 0;
if(invalid_size || !N || !batch_count)
{
DAPI_EXPECT(invalid_size ? HIPBLAS_STATUS_INVALID_VALUE : HIPBLAS_STATUS_SUCCESS,
hipblasHerBatchedFn,
(handle, uplo, N, nullptr, nullptr, incx, nullptr, lda, batch_count));
return;
}
// Naming: dK is in GPU (device) memory. hK is in CPU (host) memory
host_batch_matrix<T> hA(N, N, lda, batch_count);
host_batch_matrix<T> hA_cpu(N, N, lda, batch_count);
host_batch_matrix<T> hA_host(N, N, lda, batch_count);
host_batch_matrix<T> hA_device(N, N, lda, batch_count);
host_batch_vector<T> hx(N, incx, batch_count);
device_batch_matrix<T> dA(N, N, lda, batch_count);
device_batch_vector<T> dx(N, incx, batch_count);
device_vector<U> d_alpha(1);
// Check device memory allocation
CHECK_DEVICE_ALLOCATION(dA.memcheck());
CHECK_DEVICE_ALLOCATION(dx.memcheck());
CHECK_DEVICE_ALLOCATION(d_alpha.memcheck());
// Initial Data on CPU
hipblas_init_matrix(hA, arg, hipblas_client_never_set_nan, hipblas_hermitian_matrix, true);
hipblas_init_vector(hx, arg, hipblas_client_alpha_sets_nan, false, true);
hA_cpu.copy_from(hA);
CHECK_HIP_ERROR(dA.transfer_from(hA));
CHECK_HIP_ERROR(dx.transfer_from(hx));
CHECK_HIP_ERROR(hipMemcpy(d_alpha, &h_alpha, sizeof(U), hipMemcpyHostToDevice));
if(arg.unit_check || arg.norm_check)
{
/* =====================================================================
HIPBLAS
=================================================================== */
CHECK_HIPBLAS_ERROR(hipblasSetPointerMode(handle, HIPBLAS_POINTER_MODE_HOST));
DAPI_CHECK(hipblasHerBatchedFn,
(handle,
uplo,
N,
(U*)&h_alpha,
dx.ptr_on_device(),
incx,
dA.ptr_on_device(),
lda,
batch_count));
CHECK_HIP_ERROR(hA_host.transfer_from(dA));
CHECK_HIP_ERROR(dA.transfer_from(hA));
CHECK_HIPBLAS_ERROR(hipblasSetPointerMode(handle, HIPBLAS_POINTER_MODE_DEVICE));
DAPI_CHECK(hipblasHerBatchedFn,
(handle,
uplo,
N,
d_alpha,
dx.ptr_on_device(),
incx,
dA.ptr_on_device(),
lda,
batch_count));
CHECK_HIP_ERROR(hA_device.transfer_from(dA));
/* =====================================================================
CPU BLAS
=================================================================== */
for(size_t b = 0; b < batch_count; b++)
{
ref_her<T>(uplo, N, h_alpha, hx[b], incx, hA_cpu[b], lda);
}
// enable unit check, notice unit check is not invasive, but norm check is,
// unit check and norm check can not be interchanged their order
if(arg.unit_check)
{
unit_check_general<T>(N, N, batch_count, lda, hA_cpu, hA_host);
unit_check_general<T>(N, N, batch_count, lda, hA_cpu, hA_host);
}
if(arg.norm_check)
{
hipblas_error_host
= norm_check_general<T>('F', N, N, lda, hA_cpu, hA_host, batch_count);
hipblas_error_device
= norm_check_general<T>('F', N, N, lda, hA_cpu, hA_device, batch_count);
}
}
if(arg.timing)
{
double gpu_time_used{0};
CHECK_HIP_ERROR(dA.transfer_from(hA));
hipStream_t stream;
CHECK_HIPBLAS_ERROR(hipblasGetStream(handle, &stream));
CHECK_HIPBLAS_ERROR(hipblasSetPointerMode(handle, HIPBLAS_POINTER_MODE_DEVICE));
int runs = arg.cold_iters + arg.iters;
for(int iter = 0; iter < runs; iter++)
{
if(iter == arg.cold_iters)
gpu_time_used = get_time_us_sync(stream);
DAPI_DISPATCH(hipblasHerBatchedFn,
(handle,
uplo,
N,
d_alpha,
dx.ptr_on_device(),
incx,
dA.ptr_on_device(),
lda,
batch_count));
}
gpu_time_used = get_time_us_sync(stream) - gpu_time_used;
hipblasHerBatchedModel{}.log_args<U>(std::cout,
arg,
gpu_time_used,
her_gflop_count<T>(N),
her_gbyte_count<T>(N),
hipblas_error_host,
hipblas_error_device);
}
}